 ; Ŀ
 ;   Calamari - calendar generating program.                               
 ;   Copyright 1996, 2001 by Rocket Software Ltd.                          
 ;   A program that will eat you if you don't eat it first.                
 ; 

 ; Ŀ
 ;   Colander - the frame drawing subroutine.                              
 ; 
 (DEFUN COLANDER (pa month year column dayspr / left overfl styl blast pb
                                                daylst pos day row pc column)
 ; Ŀ
 ;   Need to set an overflow flag if the days won't fit into the grid.     
 ;   For 30 days to fit the month must begin before Saturday,              
 ;   and for 31 it must begin before Friday.                               
 ;   Subtract the starting day -1 from 35 (the number of spaces in the     
 ;   grid) to get the number of available day spaces, and subtract this    
 ;   from the number of days in the month (Dayspr).  If the result is      
 ;   positive then there are more days than spaces, so set overfl.         
 ; 
  (setq left (- 35 (1- column)))
  (setq left (- dayspr left))
  (if (minusp left)
      (setq overfl 0)
      (setq overfl left))
 ; Ŀ
 ;   First make sure the text style and width are correct.                 
 ; 
  (setq styl (tblsearch "style" "standard"))
  (if (not (and (= (strcase (cdr (assoc 3 styl))) "SIMPLEX")
                (= (cdr (assoc 41 styl)) 0.8)))
      (command "style" "standard" "simplex" 0 0.8 0 "n" "n" "n"))
 ; Ŀ
 ;   Draw the top line, make it into an array six deep for the day         
 ;   squares, copy it up half an inch for the day name boxes.              
 ; 
  (command "line" pa (polar pa 0 7) "")
  (setq blast (entlast))
  (command "array" blast "" "r" 6 1 -1)
  (command "copy" blast "" pa (setq pb (polar pa (/ pi 2) 0.5)))
 ; Ŀ
 ;   Draw a line from the top of the day name boxes down 5.5" , array it   
 ;   seven times for the day square sides.                                 
 ; 
  (command "line" pb (polar pa (* pi 1.5) 5) "")
  (setq blast (entlast))
  (command "array" blast "" "r" 1 7 1)
 ; Ŀ
 ;   Extend the line 1/2" upwards, copy it to the right end of the grid.   
 ; 
  (command "change" blast "" (polar pa (/ pi 2) 1))
  (command "copy" blast "" pa (polar pa 0 7))
 ; Ŀ
 ;   Draw the top line of the grid, forming the month and year text box.   
 ; 
  (command "line" (setq pb (polar pa (/ pi 2) 1)) (polar pb 0 7) "")
 ; Ŀ
 ;   Find the middle of the first weekday name box.                        
 ; 
  (setq pb (polar (polar pa 0 0.5) (/ pi 2) 0.25))
 ; Ŀ
 ;   Make the list of weekdays.  Set the position in the list to 0.        
 ; 
  (setq daylst (list "Sunday"   "Monday" "Tuesday" "Wednesday"
                     "Thursday" "Friday" "Saturday"))
  (setq pos 0)
 ; Ŀ
 ;   Write the names in order into the boxes.                              
 ; 
  (while (setq day (nth pos daylst))
         (command "text" "m" pb 0.125 0 day)
         (setq pos (1+ pos))
         (setq pb (polar pb 0 1)))
 ; Ŀ
 ;   Write the year and month at the appropriate location.                 
 ;   Might want to capitalise this.                                        
 ; 
  (setq pb (polar (polar pa (/ pi 2) 0.75) 0 3.5))
  (command "text" "m" pb 0.2 0 (strcat month " " year))
 ; Ŀ
 ;   Locate the box containing the first day of the month.                 
 ; 
  (setq pb (polar pa 0 (1- column)))
  (setq day 1)
 ; Ŀ
 ;   Number the boxes.                                                     
 ;   Pb should now be the upper left corner of the current day box.        
 ; 
  (setq row 1)
  (while (<= day dayspr)
 ; Ŀ
 ;   Cond: we are in the last row and there are overflow days.             
 ; 
         (cond ((and (= row 5) (> overfl 0))
                (command "line" (polar pb (* pi 1.5) 1)
                                (polar pb 0 1) "")
                (setq pc (polar (polar pb (* pi 1.5) 0.25) 0 0.25))
                (command "text" "m" pc 0.25 0 (itoa day))
                (setq overfl (1- overfl)))
 ; Ŀ
 ;   Cond: we are past the last row, so install overflow numbers in        
 ;   the 5th row.                                                          
 ; 
               ((= row 6)
                (setq pc (polar (polar pb (* pi 0.5) 0.25) 0 0.75))
                (command "text" "m" pc 0.25 0 (itoa day)))
 ; Ŀ
 ;   Cond: the default: this is a normal day, so insert a comment (if      
 ;   the random comment finder will spit one out) and write the date.      
 ; 
             (T
              (setq datlen (* (length callst) 3))
              (setq prev (r100 1))
              (setq strnum (fix (* datlen prev)))
              (setq txtstr "")
              (while (and (or (= txtstr "") (null txtstr))
                          (<= strnum datlen))
                     (setq asub (assoc (1+ strnum) callst))
                     (setq txtstr (cdr asub))
                     (setq strnum (1+ strnum)))
              (if asub (setq callst (subst (cons strnum "") asub callst)))
              (if txtstr (tins pb txtstr))
 ; Ŀ
 ;   Locate the day number text insertion and write the day.               
 ; 
              (setq pc (polar (polar pb (* pi 1.5) 0.25) 0 0.25))
              (command "text" "m" pc 0.25 0 (itoa day))))
 ; Ŀ
 ;   Cond end.  Go to the next date and the next square.                   
 ; 
         (setq day (1+ day))
         (setq column (1+ column))
 ; Ŀ
 ;   If column (the weekday number) is > 7 then move back to the first     
 ;   column and down one row, otherwise move one column to the right.      
 ; 
         (if (> column 7)
             (progn
                  (setq row (1+ row))
                  (setq column 1)
                  (setq pb (list (car pa) (cadr (polar pb (* pi 1.5) 1)))))
             (setq pb (polar pb 0 1))))
 (princ))
 ; Ŀ
 ;   Colander end.                                                         
 ; 

 ; Ŀ
 ;   Orca - error handler.                                                 
 ; 
 (DEFUN ORCA (shk / pos entt enam sublst vall)
  (setq *error* esav)
  (if (/= shk "Function cancelled") (write-line shk))
 (princ))
 ; Ŀ
 ;   Orca end.                                                             
 ; 

 ; Ŀ
 ;   R100 - Random Number Generator.                                       
 ; 
 (DEFUN R100 (prev / s t1)
  (setq t1 prev)
  (while (= prev t1)
         (setq s (* (getvar "cdate") 10000000.0))
         (setq t1 (- s ( fix s))))
(print t1)
 t1)
 ; Ŀ
 ;   R100 end.                                                             
 ; 

 ; Ŀ
 ;   Fork - read the random date text file into an association list.       
 ;   May be rewritten to take a filename as an argument and return a list. 
 ; 
 (DEFUN FORK (/ fn fnam callst num aa)
  (setq callst (list (list "a" "a")))
 ; Ŀ
 ;   Find the data file, or ask for its location.                          
 ; 
  (if (null (setq fnam (findfile "calendar.dat")))
      (setq fnam (getfiled "Calendar data file Calendar.dat" "" "" 0)))
  (setq fn (open fnam "r"))
  (setq num 0)
  (while (setq aa (read-line fn))
         (setq callst (append callst (list (cons num aa))))
         (setq num (1+ num)))
  (close fn)
 callst)
 ; Ŀ
 ;   Fork end.                                                             
 ; 

 ; Ŀ
 ;   Subroutine Daz: find the weekday matching a given date.               
 ;   Takes three argument strings: yr, the year                            
 ;                                 mm, the month                           
 ;                                 days, the day of the month              
 ;   Returns a list: the day of the week and the number of days in the     
 ;   month.                                                                
 ;                                                                         
 ;   Are you sure you should be playing with this?                         
 ; 
 (DEFUN DAZ (yr mm days / yr mm days nn months dayspr offset years jan1st)
  (setq yr (read yr))
  (setq mm (1- (read mm)))
  (setq days (read days))
  (setq nn (rem (if (> 0 yr) (1+ yr) yr) 4))
  (setq months (list 31 (if (= nn 0) 29 28) 31 30 31 30 31 31 30 31 30 31))
  (setq dayspr (nth mm months))
  (while (> mm 0)
         (setq days (+ days (nth (1- mm) months)))
         (setq mm (1- mm)))
  (setq offset (rem (- yr 13) 28))
  (setq years (list 0 1 2 3 5 6 0 1 3 4 5 6 1 2 3 4 6 0 1 2 4 5 6 0 2 3 4 5))
  (setq jan1st (1- (+ days (nth offset years))))
  (setq jan1st (rem jan1st 7))
  (setq days (list "Mon" "Tues" "Wednes" "Thurs" "Fri" "Satur" "Sun"))
  (list (strcat (nth jan1st days) "day") dayspr))
 ; Ŀ
 ;   Daz end.                                                              
 ; 


 ; Ŀ
 ;   Subroutine Tentacle - squeeze text to a real width of 0.95.           
 ;   Takes one argument: a text ename.  Returns nothing.                   
 ; 
 (DEFUN TENTACLE (enam / bb cc dd realwd widscl prev41 scalfc)
  (setq bb (textbox (setq entt (entget enam))))
  (setq cc (car bb))                    ; ll offset from 10 of text
  (setq dd (cadr bb))                   ; ur offset from 10 of text
  (setq realwd (- (car dd) (car cc)))
 ; Ŀ
 ;   Find the text width scale factor.                                     
 ; 
  (setq widscl (cdr (setq prev41 (assoc 41 entt))))
 ; Ŀ
 ;   Compare the actual and desired widths.  If the actual is more than    
 ;   the desired width, then adjust the width scale factor to make it fit. 
 ; 
  (if (> realwd 0.95)
      (progn
           (setq scalfc (/ 0.95 realwd))
           (setq widscl (* widscl scalfc))
           (entmod (subst (cons 41 widscl) prev41 entt)))))
 ; Ŀ
 ;   Tentacle end.                                                         
 ; 


 ; Ŀ
 ;   Subroutine Splat - divide a text string at each occurrence of a       
 ;   given character, make into a list of substrings.                      
 ;   Arguments: Sepchr, the field separator character.                     
 ;              Linn, the text string.                                     
 ;   Returns a list of field values, removes leading and trailing spaces.  
 ; 
 (DEFUN SPLAT (sepchr linn / len pos name1 strlst)
  (while (/= (strlen linn) 0)
         (while (and (= (substr linn 1 1) " ")
                     (/= (strlen linn) 0))
                (setq linn (substr linn 2)))
         (while (= (substr linn (setq len (strlen linn))) " ")
                (setq linn (substr linn 1 (1- len))))
         (setq pos 1)
         (setq len (strlen linn))
         (while (and (/= (substr linn pos 1) sepchr)
                     (>= len pos))
                (setq pos (1+ pos)))
         (setq name1 (substr linn 1 (1- pos)))
         (while (= (substr name1 (setq len (strlen linn))) " ")
                (setq name1 (substr name1 1 (1- len))))
         (setq linn (substr linn (1+ pos)))
         (setq strlst (append strlst (list name1))))
  (if (null strlst) (setq strlst (list "")))
  strlst)
 ; Ŀ
 ;   Splat end.                                                            
 ; 

 ; Ŀ
 ;   Subroutine Txtlen - find the length drawing units of a text entity    
 ;   containing a given string.                                            
 ;   Takes two arguments - the string and the text width factor.           
 ;   Assumes a text height of 0.05".                                       
 ;   Returns a length.                                                     
 ; 
 (DEFUN TXTLEN (str widfac /)
  (setq tex (list (cons 1 str) (cons 40 0.05) (cons 41 widfac)))
  (setq bb (textbox tex))
  (setq cc (car bb))                    ; ll offset from 10 of text
  (setq dd (cadr bb))                   ; ur offset from 10 of text
  (setq len (- (car dd) (car cc))))
 ; Ŀ
 ;   Txtlen end                                                            
 ; 

 ; Ŀ
 ;   Tins - draw the comment text stack.                                   
 ; 
 (DEFUN TINS (pa str / strlst pos topstr sub prnlst pb)
  (setq strlst (splat " " str))
  (setq pos 0)
  (setq topstr "")
  (while (setq sub (nth pos strlst))
         (setq pos (1+ pos))
         (setq allstr (strcat topstr (if (= topstr "") "" " ") sub))
         (if (<= (txtlen allstr 0.8) 0.95)
             (setq topstr allstr)
             (progn
                  (setq prnlst (append prnlst (list topstr)))
                  (setq topstr sub))))
 ; Ŀ
 ;   Add the final topstr to the list.                                     
 ; 
  (setq prnlst (append prnlst (list topstr)))
 ; Ŀ
 ;   Find the text stack height.                                           
 ; 
  (setq depth (+ 0.025 (* (1- (length prnlst)) 0.07)))
  (setq pb (polar (polar pa (* pi 1.5) (- 1 depth)) 0 0.025))
  (while (setq str (car prnlst))
         (command "text" pb 0.05 0 (car prnlst))
         (tentacle (entlast))
         (setq prnlst (cdr prnlst))
         (setq pb (polar pb (* pi 1.5) 0.07)))
 (princ))
 ; Ŀ
 ;   Tins end.                                                             
 ; 

 ; Ŀ
 ;   Calamari - the calendar maker.                                        
 ; 
 (DEFUN C:CALAMARI (/ date year monthp molist month months mono yearp monum
                                           pa num day1 dayspr daylst daynum)
  (setvar "cmdecho" 0)
  (command "undo" "m")
  (setq esav *error*)
  (setq *error* orca)
  (setvar "blipmode" 0)
  (setvar "limcheck" 0)
 ; Ŀ
 ;   Read the random data file into the list callst.                       
 ; 
  (setq callst (fork))
 ; Ŀ
 ;   Get the date, extract month and year for use in defaults, deduce      
 ;   the name of the month.                                                
 ; 
  (setq date (rtos (getvar "cdate") 2 12))
  (setq year (substr date 1 4))
  (setq monthp (1- (read (substr date 5 2))))
  (setq monthp (nth monthp (setq molist (list "January" "February" "March"
                                              "April" "May" "June" "July"
                                              "August" "September" "October"
                                              "November" "December"))))
 ; Ŀ
 ;   Ask for the month to begin the calendar with, convert the response    
 ;   to the proper capitalised form and save it in the variable Month,     
 ;   save the number of the month in the variable mono.                    
 ; 
  (initget 0 (strcat "JAnuary February MARch APril MAY JUNe JULy "
                     "AUgust September October November December"))
  (Setq month (getkword (strcat "\nStarting Month <" monthp ">: ")))
  (if month
      (setq month (strcat (strcase (substr month 1 1))
                          (strcase (substr month 2) t)))
      (setq month monthp))
  (setq months '(("January" "01") ("February" "02") ("March" "03")
                 ("April" "04")   ("May" "05")      ("June" "06")
                 ("July" "07")    ("August" "08")   ("September" "09")
                 ("October" "10") ("November" "11") ("December" "12")))
  (setq mono (cadr (assoc month months)))
 ; Ŀ
 ;   Ask for the year of the calendar.  Don't allow a year 0.              
 ; 
  (setq yearp (getstring (strcat "\nYear <" year ">: ")))
  (if (= yearp "0")
      (progn
           (prompt "There was no year 0, bonehead.")
           (setq yearp (getstring (strcat "\nYear <" year ">: ")))))
  (if (/= yearp "") (setq year yearp))
 ; Ŀ
 ;   Ask for the number of months to draw.                                 
 ; 
  (setq monum (getint "\nNumber of months <12>: "))
  (if (null monum) (setq monum 12))
 ; Ŀ
 ;   And the number of columns to arrange them in.                         
 ; 
  (cond ((= monum 12)
         (setq colnum 4))
        ((> monum 12)
         (setq colnum 12))
        ((< monum 3)
         (setq colnum (fix (sqrt monum))))
        ((< monum 12)
         (setq colnum (fix (1+ (sqrt monum))))))
  (if (= monum 1)
      (setq colnum 1)
      (progn
           (setq colnump (getint (strcat "\nNumber of columns <"
                                         (itoa colnum) ">: ")))
           (if colnump (setq colnum colnump))))
 ; Ŀ
 ;   Draw the months.                                                      
 ; 
  (setq pa (getpoint "Top left: "))
  (setq num 0)
  (repeat monum
         (setq num (1+ num))
 ; Ŀ
 ;   Find the weekday on which the first day of the given month/year       
 ;   falls.  The returned list will contain the weekday and the number     
 ;   of days in the month.                                                 
 ; 
         (setq day1 (daz year mono "01"))
 ; Ŀ
 ;   Variables: Dayspr - the number of days in the month.                  
 ;              Day1   - the weekday of the first of the month.            
 ;              column - the number of the weekday.                        
 ; 
         (setq dayspr (cadr day1))
         (setq day1 (car day1))
         (setq daylst '(("Sunday" 1) ("Monday" 2) ("Tuesday" 3) ("Wednesday" 4)
                        ("Thursday" 5) ("Friday" 6) ("Saturday" 7)))
         (setq daynum (cadr (assoc day1 daylst)))
 ; Ŀ
 ;   Now call Cal to draw the months.                                      
 ; 
         (colander pa month year daynum dayspr)
 ; Ŀ
 ;   Increment the position.                                               
 ; 
         (if (= num colnum)
             (progn
                  (setq pa (polar pa pi (* (1- colnum) 8)))
                  (setq pa (polar pa (* pi 1.5) 7))
                  (setq num 0))
             (setq pa (polar pa 0 8)))
 ; Ŀ
 ;   And the month and year.                                               
 ; 
         (if (= month "December")
             (progn
                  (setq mono "1")
                  (setq year (itoa (1+ (read year)))))
             (setq mono (itoa (1+ (read mono)))))
         (setq month (nth (1- (read mono)) molist)))
  (setq callst ())
  (setq *error* esav)
 (princ))