 ; Ŀ
 ;   Dent - search and replace for attribute prompts in the block tables.  
 ;   Copyright 1996 by Rocket Software                                     
 ;   Special cases: 1. If %% is input for the existing string then Dent    
 ;                     doesn't ask for a replacement string and the first  
 ;                     letter in all prompts in the selected block         
 ;                     definitions is converted to upper case.             
 ;                  2. If %% is input for the replacement string then      
 ;                     the string is assumed to be empty.                  
 ;                                                                         
 ;   Like a .44 magnum that can only be fired at poodles, this routine     
 ;   is powerful, dangerous in the wrong hands, and difficult to see any   
 ;   legitimate use for.  Then again, some things are irritating enough    
 ;   to make the risk acceptable.                                          
 ;                                                                         
 ; 

 ; Ŀ
 ;   Chug - string substitution engine.  Takes the search string, the      
 ;   replacement string, and the target string as arguments, and returns   
 ;   the (possibly modified) target string and the number of changes made. 
 ; 
 (DEFUN CHUG (oldstr newstr exstr / pos chnum changd newlen chunk)
  (setq pos 1)
  (setq chnum 0)
  (setq changd ())
  (setq newlen (strlen newstr))
  (setq oldlen (strlen oldstr))
  (while (= oldlen (strlen (setq chunk (substr exstr pos oldlen))))
         (if (= chunk oldstr)
             (progn
                  (setq exstr (strcat (substr exstr 1 (1- pos))
                                       newstr
                                      (substr exstr (+ pos oldlen))))
                  (setq changd t)
                  (setq chnum (1+ chnum))
                  (setq pos (+ pos newlen)))
             (setq pos (1+ pos))))
 (list exstr chnum))
 ; Ŀ
 ;   Chug - end.                                                           
 ; 

 ; Ŀ
 ;   Hug - string capitaliser.  Takes one argument, a string, and returns  
 ;   a list: the string with the first letter changed to upper case and    
 ;   T if this changed the string, () if not.                              
 ; 
 (DEFUN HUG (exstr / chunk1 chunka)
  (setq chunk1 (substr exstr 1 1))
  (if (= chunk1 (setq chunka (strcase chunk1)))
      (list exstr ())
      (list (strcat chunka (substr exstr 2)) T)))
 ; Ŀ
 ;   Hug - end.                                                            
 ; 

 ; Ŀ
 ;   Rust - change a prompt string in the block tables.                    
 ;   Notes: 1. Entnext returns nil after the last entity in a block        
 ;             definition.                                                 
 ;          2. An empty block has one subentity of type Endblk.            
 ;   Takes 3 arguments - the first subentity ename, the search string      
 ;   and the replacement string.                                           
 ;   Returns a list of the number of prompt strings changed and the        
 ;   total number of changes.                                              
 ; 
 (DEFUN RUST (namm oldstr newstr / attchg chgnum entt prom1 prom2 altr mods
                                                                        nup)
  (setq attchg 0)
  (setq chgnum 0)
  (while (and namm (setq entt (entget namm)))            ; the whole thing
         (if (setq prom1 (assoc 3 entt))
             (progn
                  (setq prom2 (cdr prom1))
                  (setq altr (chug oldstr newstr prom2))
                  (if (> (setq mods (cadr altr)) 0)      ; if any changes made
                      (progn
                           (setq nup (car altr))
                           (entmod (subst (cons 3 nup) prom1 entt))
                           (setq attchg (1+ attchg))
                           (setq chgnum (+ chgnum mods))))))
         (setq namm (entnext namm)))                     ; next subentity ename
 (list attchg chgnum))
 ; Ŀ
 ;   Rust end.                                                             
 ; 

 ; Ŀ
 ;   Strum - capitalise a prompt string in the block tables.               
 ;   Takes one argument - the first subentity ename.                       
 ;   Returns a list of the number of prompt strings changed and the        
 ;   total number of changes (which will be the same).                     
 ; 
 (DEFUN STRUM (namm / attchg entt prom1 nup)
  (setq attchg 0)
  (while (and namm (setq entt (entget namm)))            ; the whole thing
         (if (setq prom1 (assoc 3 entt))
             (progn
                  (setq nup (hug (cdr prom1)))
                  (if (cadr nup)
                      (progn
                           (entmod (subst (cons 3 (car nup)) prom1 entt))
                           (setq attchg (1+ attchg))))))
         (setq namm (entnext namm)))                     ; next subentity ename
 (list attchg attchg))
 ; Ŀ
 ;   Strum end.                                                            
 ; 

 ; Ŀ
 ;   Dent.                                                                 
 ; 
 (DEFUN C:DENT (/ ach bch tchs ss num enam namm namlst oldp newp blokdat
                                                        first blnam chgs reww)
  (setq ach 0)         ; number of attdefs changed
  (setq bch 0)         ; number of block definitions changed
  (setq tchs 0)        ; total number of changes
 ; Ŀ
 ;   Get blocks to change.                                                 
 ; 
  (prompt "Select examples of blocks to change or <Enter> for all blocks: ")
  (setq ss (ssget (list (cons 0 "insert") (cons 66 1))))
  (if (null ss) (prompt "\nAll blocks will be modified."))
  (setq num 0)
  (while (and ss (setq enam (ssname ss num)))
         (setq num (1+ num))
         (setq namm (cdr (assoc 2 (entget enam))))
         (if (not (member namm namlst))
             (setq namlst (append namlst (list namm)))))
 ; Ŀ
 ;   Get old and new prompt strings.                                       
 ; 
  (if (/= (type oldstr) 'STR)
      (setq oldstr "Enter "))
  (if (/= (type newstr) 'STR)
      (setq newstr ""))
  (setq oldp (getstring t (strcat "\nString to replace <" oldstr ">: ")))
  (if (/= oldp "") (setq oldstr oldp))
  (if (/= oldstr "%%")
      (progn
           (setq newp (getstring t (strcat "\nNew string <" newstr ">: ")))
           (if (/= newp "") (setq newstr newp))
           (if (= newstr "%%") (setq newstr ""))))
 ; Ŀ
 ;   Change the selected blocks, or all blocks if none were selected.      
 ; 
  (if namlst
     (progn
          (while (setq namm (car namlst))
                 (setq namlst (cdr namlst))
                 (setq blokdat (tblsearch "block" namm))
                 (setq first (substr (setq blnam (cdr (assoc 2 blokdat))) 1 1))
                 (grtext -2 blnam)
                 (if (/= first "*")
                     (progn
                          (setq namm (cdr (assoc -2 blokdat)))
                          (if (= oldstr "%%")
                              (setq chgs (strum namm))
                              (setq chgs (rust namm oldstr newstr)))
                          (if (> (car chgs) 0) (setq bch (1+ bch)))
                          (setq ach (+ ach (car chgs)))
                          (setq tchs (+ tchs (cadr chgs)))))))
     (progn
          (setq reww T)
          (while (setq blokdat (tblnext "block" reww))
                 (setq reww ())
                 (setq first (substr (setq blnam (cdr (assoc 2 blokdat))) 1 1))
                 (grtext -2 blnam)
                 (if (/= first "*")
                     (progn
                          (setq namm (cdr (assoc -2 blokdat)))
                          (if (= oldstr "%%")
                              (setq chgs (strum namm))
                              (setq chgs (rust namm oldstr newstr)))
                          (if (> (car chgs) 0) (setq bch (1+ bch)))
                          (setq ach (+ ach (car chgs)))
                          (setq tchs (+ tchs (cadr chgs))))))))
 ; Ŀ
 ;   Sum up and end.                                                       
 ; 
  (write-line (strcat (itoa tchs) " change" (if (/= tchs 1) "s" "") " in "
                      (itoa ach) " prompt"  (if (/= ach 1) "s" "") " in "
                      (itoa bch) " block definition" (if (/= bch 1) "s." ".")))
 (princ))